@@ -1,6 +1,8 @@
 //! Windows and macOS-specific functions regarding CPU usage.
 
-pub fn convert_cpu_times(cpu_time: &heim::cpu::CpuTime) -> (f64, f64) {
+use crate::components::tui_widget::time_chart::Point;
+
+pub fn convert_cpu_times(cpu_time: &heim::cpu::CpuTime) -> Point {
     let working_time: f64 =
         (cpu_time.user() + cpu_time.system()).get::<heim::units::time::second>();
     (
