@@ -1,6 +1,7 @@
-use itertools::izip;
-use std::{collections::HashMap, str::FromStr};
+use std::str::FromStr;
 
+use canvas_colours::*;
+use itertools::izip;
 use tui::{
     backend::Backend,
     layout::{Constraint, Direction, Layout, Rect},
@@ -9,13 +10,6 @@ use tui::{
     Frame, Terminal,
 };
 
-// use ordered_float::OrderedFloat;
-
-use canvas_colours::*;
-use dialogs::*;
-use screens::*;
-use widgets::*;
-
 use crate::{
     app::{
         self,
@@ -23,46 +17,15 @@ use crate::{
         App,
     },
     constants::*,
-    data_conversion::{ConvertedBatteryData, ConvertedCpuData, ConvertedProcessData},
-    options::Config,
     utils::error,
     utils::error::BottomError,
-    Pid,
 };
 
-mod canvas_colours;
+pub mod canvas_colours;
 mod dialogs;
 mod drawing_utils;
-mod screens;
 mod widgets;
 
-/// Point is of time, data
-type Point = (f64, f64);
-
-#[derive(Default)]
-pub struct DisplayableData {
-    pub rx_display: String,
-    pub tx_display: String,
-    pub total_rx_display: String,
-    pub total_tx_display: String,
-    pub network_data_rx: Vec<Point>,
-    pub network_data_tx: Vec<Point>,
-    pub disk_data: Vec<Vec<String>>,
-    pub temp_sensor_data: Vec<Vec<String>>,
-    pub single_process_data: HashMap<Pid, ConvertedProcessData>, // Contains single process data, key is PID
-    pub finalized_process_data_map: HashMap<u64, Vec<ConvertedProcessData>>, // What's actually displayed, key is the widget ID.
-    pub stringified_process_data_map: HashMap<u64, Vec<(Vec<(String, Option<String>)>, bool)>>, // Represents the row and whether it is disabled, key is the widget ID
-
-    pub mem_labels: Option<(String, String)>,
-    pub swap_labels: Option<(String, String)>,
-
-    pub mem_data: Vec<Point>, // TODO: Switch this and all data points over to a better data structure...
-    pub swap_data: Vec<Point>,
-    pub load_avg_data: [f32; 3],
-    pub cpu_data: Vec<ConvertedCpuData>,
-    pub battery_data: Vec<ConvertedBatteryData>,
-}
-
 #[derive(Debug)]
 pub enum ColourScheme {
     Default,
@@ -94,27 +57,32 @@ impl FromStr for ColourScheme {
     }
 }
 
-/// Handles the canvas' state.  TODO: [OPT] implement this.
+/// Handles the canvas' state.
 pub struct Painter {
     pub colours: CanvasColours,
     height: u16,
     width: u16,
     styled_help_text: Vec<Spans<'static>>,
-    is_mac_os: bool, // FIXME: This feels out of place...
-    row_constraints: Vec<Constraint>,
-    col_constraints: Vec<Vec<Constraint>>,
-    col_row_constraints: Vec<Vec<Vec<Constraint>>>,
-    layout_constraints: Vec<Vec<Vec<Vec<Constraint>>>>,
+    is_mac_os: bool, // TODO: This feels out of place...
+
+    // TODO: Redo this entire thing.
+    row_constraints: Vec<LayoutConstraint>,
+    col_constraints: Vec<Vec<LayoutConstraint>>,
+    col_row_constraints: Vec<Vec<Vec<LayoutConstraint>>>,
+    layout_constraints: Vec<Vec<Vec<Vec<LayoutConstraint>>>>,
     derived_widget_draw_locs: Vec<Vec<Vec<Vec<Rect>>>>,
     widget_layout: BottomLayout,
-    table_height_offset: u16,
+}
+
+// Part of a temporary fix for https://github.com/ClementTsang/bottom/issues/896
+enum LayoutConstraint {
+    CanvasHandled,
+    Grow,
+    Ratio(u32, u32),
 }
 
 impl Painter {
-    pub fn init(
-        widget_layout: BottomLayout, table_gap: u16, is_basic_mode: bool, config: &Config,
-        colour_scheme: ColourScheme,
-    ) -> anyhow::Result<Self> {
+    pub fn init(widget_layout: BottomLayout, colours: CanvasColours) -> anyhow::Result<Self> {
         // Now for modularity; we have to also initialize the base layouts!
         // We want to do this ONCE and reuse; after this we can just construct
         // based on the console size.
@@ -126,9 +94,9 @@ impl Painter {
 
         widget_layout.rows.iter().for_each(|row| {
             if row.canvas_handle_height {
-                row_constraints.push(Constraint::Length(0));
+                row_constraints.push(LayoutConstraint::CanvasHandled);
             } else {
-                row_constraints.push(Constraint::Ratio(
+                row_constraints.push(LayoutConstraint::Ratio(
                     row.row_height_ratio,
                     widget_layout.total_row_height_ratio,
                 ));
@@ -139,21 +107,23 @@ impl Painter {
             let mut new_col_row_constraints = Vec::new();
             row.children.iter().for_each(|col| {
                 if col.canvas_handle_width {
-                    new_col_constraints.push(Constraint::Length(0));
+                    new_col_constraints.push(LayoutConstraint::CanvasHandled);
                 } else {
-                    new_col_constraints
-                        .push(Constraint::Ratio(col.col_width_ratio, row.total_col_ratio));
+                    new_col_constraints.push(LayoutConstraint::Ratio(
+                        col.col_width_ratio,
+                        row.total_col_ratio,
+                    ));
                 }
 
                 let mut new_new_col_row_constraints = Vec::new();
                 let mut new_new_widget_constraints = Vec::new();
                 col.children.iter().for_each(|col_row| {
                     if col_row.canvas_handle_height {
-                        new_new_col_row_constraints.push(Constraint::Length(0));
+                        new_new_col_row_constraints.push(LayoutConstraint::CanvasHandled);
                     } else if col_row.flex_grow {
-                        new_new_col_row_constraints.push(Constraint::Min(0));
+                        new_new_col_row_constraints.push(LayoutConstraint::Grow);
                     } else {
-                        new_new_col_row_constraints.push(Constraint::Ratio(
+                        new_new_col_row_constraints.push(LayoutConstraint::Ratio(
                             col_row.col_row_height_ratio,
                             col.total_col_row_ratio,
                         ));
@@ -162,11 +132,11 @@ impl Painter {
                     let mut new_new_new_widget_constraints = Vec::new();
                     col_row.children.iter().for_each(|widget| {
                         if widget.canvas_handle_width {
-                            new_new_new_widget_constraints.push(Constraint::Length(0));
+                            new_new_new_widget_constraints.push(LayoutConstraint::CanvasHandled);
                         } else if widget.flex_grow {
-                            new_new_new_widget_constraints.push(Constraint::Min(0));
+                            new_new_new_widget_constraints.push(LayoutConstraint::Grow);
                         } else {
-                            new_new_new_widget_constraints.push(Constraint::Ratio(
+                            new_new_new_widget_constraints.push(LayoutConstraint::Ratio(
                                 widget.width_ratio,
                                 col_row.total_widget_ratio,
                             ));
@@ -183,7 +153,7 @@ impl Painter {
         });
 
         let mut painter = Painter {
-            colours: CanvasColours::default(),
+            colours,
             height: 0,
             width: 0,
             styled_help_text: Vec::default(),
@@ -194,58 +164,21 @@ impl Painter {
             layout_constraints,
             widget_layout,
             derived_widget_draw_locs: Vec::default(),
-            table_height_offset: if is_basic_mode { 2 } else { 4 } + table_gap,
         };
 
-        if let ColourScheme::Custom = colour_scheme {
-            painter.generate_config_colours(config)?;
-        } else {
-            painter.generate_colour_scheme(colour_scheme)?;
-        }
         painter.complete_painter_init();
 
         Ok(painter)
     }
 
-    fn generate_config_colours(&mut self, config: &Config) -> anyhow::Result<()> {
-        if let Some(colours) = &config.colors {
-            self.colours.set_colours_from_palette(colours)?;
+    /// Determines the border style.
+    pub fn get_border_style(&self, widget_id: u64, selected_widget_id: u64) -> tui::style::Style {
+        let is_on_widget = widget_id == selected_widget_id;
+        if is_on_widget {
+            self.colours.highlighted_border_style
+        } else {
+            self.colours.border_style
         }
-
-        Ok(())
-    }
-
-    fn generate_colour_scheme(&mut self, colour_scheme: ColourScheme) -> anyhow::Result<()> {
-        match colour_scheme {
-            ColourScheme::Default => {
-                // Don't have to do anything.
-            }
-            ColourScheme::DefaultLight => {
-                self.colours
-                    .set_colours_from_palette(&*DEFAULT_LIGHT_MODE_COLOUR_PALETTE)?;
-            }
-            ColourScheme::Gruvbox => {
-                self.colours
-                    .set_colours_from_palette(&*GRUVBOX_COLOUR_PALETTE)?;
-            }
-            ColourScheme::GruvboxLight => {
-                self.colours
-                    .set_colours_from_palette(&*GRUVBOX_LIGHT_COLOUR_PALETTE)?;
-            }
-            ColourScheme::Nord => {
-                self.colours
-                    .set_colours_from_palette(&*NORD_COLOUR_PALETTE)?;
-            }
-            ColourScheme::NordLight => {
-                self.colours
-                    .set_colours_from_palette(&*NORD_LIGHT_COLOUR_PALETTE)?;
-            }
-            ColourScheme::Custom => {
-                // This case should never occur, just do nothing.
-            }
-        }
-
-        Ok(())
     }
 
     /// Must be run once before drawing, but after setting colours.
@@ -301,7 +234,7 @@ impl Painter {
         use BottomWidgetType::*;
 
         terminal.draw(|f| {
-            let (terminal_size, frozen_draw_loc) = if app_state.is_frozen {
+            let (terminal_size, frozen_draw_loc) = if app_state.frozen_state.is_frozen() {
                 let split_loc = Layout::default()
                     .constraints([Constraint::Min(0), Constraint::Length(1)])
                     .split(f.size());
@@ -334,12 +267,6 @@ impl Painter {
                 for battery_widget in app_state.battery_state.widget_states.values_mut() {
                     battery_widget.tab_click_locs = None;
                 }
-
-                // Reset column headers for sorting in process widget...
-                for proc_widget in app_state.proc_state.widget_states.values_mut() {
-                    proc_widget.columns.column_header_y_loc = None;
-                    proc_widget.columns.column_header_x_locs = None;
-                }
             }
 
             if app_state.help_dialog_state.is_showing_help {
@@ -365,21 +292,15 @@ impl Painter {
                         ]
                     } else {
                         [
-                            Constraint::Percentage(20),
-                            Constraint::Percentage(60),
-                            Constraint::Percentage(20),
+                            Constraint::Percentage(15),
+                            Constraint::Percentage(70),
+                            Constraint::Percentage(15),
                         ]
                     })
                     .split(vertical_dialog_chunk[1]);
 
                 self.draw_help_dialog(f, app_state, middle_dialog_chunk[1]);
             } else if app_state.delete_dialog_state.is_showing_dd {
-                // TODO: This needs the paragraph wrap feature from tui-rs to be pushed to complete... but for now it's pretty close!
-                // The main problem right now is that I cannot properly calculate the height offset since
-                // line-wrapping is NOT the same as taking the width of the text and dividing by width.
-                // So, I need the height AFTER wrapping.
-                // See: https://github.com/fdehau/tui-rs/pull/349.  Land this after this pushes to release.
-
                 let dd_text = self.get_dd_spans(app_state);
 
                 let text_width = if terminal_width < 100 {
@@ -396,37 +317,6 @@ impl Painter {
                     22
                 };
 
-                // let (text_width, text_height) = if let Some(dd_text) = &dd_text {
-                //     let width = if current_width < 100 {
-                //         current_width * 90 / 100
-                //     } else {
-                //         let min_possible_width = (current_width * 50 / 100) as usize;
-                //         let mut width = dd_text.width();
-
-                //         // This should theoretically never allow width to be 0... we can be safe and do an extra check though.
-                //         while width > (current_width as usize) && width / 2 > min_possible_width {
-                //             width /= 2;
-                //         }
-
-                //         std::cmp::max(width, min_possible_width) as u16
-                //     };
-
-                //     (
-                //         width,
-                //         (dd_text.height() + 2 + (dd_text.width() / width as usize)) as u16,
-                //     )
-                // } else {
-                //     // AFAIK this shouldn't happen, unless something went wrong...
-                //     (
-                //         if current_width < 100 {
-                //             current_width * 90 / 100
-                //         } else {
-                //             current_width * 50 / 100
-                //         },
-                //         7,
-                //     )
-                // };
-
                 let vertical_bordering = terminal_height.saturating_sub(text_height) / 2;
                 let vertical_dialog_chunk = Layout::default()
                     .direction(Direction::Vertical)
@@ -477,14 +367,12 @@ impl Painter {
                         f,
                         app_state,
                         rect[0],
-                        true,
                         app_state.current_widget.widget_id,
                     ),
                     Temp => self.draw_temp_table(
                         f,
                         app_state,
                         rect[0],
-                        true,
                         app_state.current_widget.widget_id,
                     ),
                     Net => self.draw_network_graph(
@@ -502,7 +390,7 @@ impl Painter {
                                 _ => 0,
                             };
 
-                        self.draw_process_features(f, app_state, rect[0], true, widget_id);
+                        self.draw_process_widget(f, app_state, rect[0], true, widget_id);
                     }
                     Battery => self.draw_battery_display(
                         f,
@@ -513,13 +401,6 @@ impl Painter {
                     ),
                     _ => {}
                 }
-            } else if app_state.is_config_open {
-                let rect = Layout::default()
-                    .margin(0)
-                    .constraints([Constraint::Percentage(100)])
-                    .split(f.size())[0];
-
-                self.draw_config_screen(f, app_state, rect)
             } else if app_state.app_config_fields.use_basic_mode {
                 // Basic mode.  This basically removes all graphs but otherwise
                 // the same info.
@@ -527,12 +408,12 @@ impl Painter {
                     self.draw_frozen_indicator(f, frozen_draw_loc);
                 }
 
-                let actual_cpu_data_len = app_state.canvas_data.cpu_data.len().saturating_sub(1);
+                let actual_cpu_data_len = app_state.converted_data.cpu_data.len().saturating_sub(1);
 
                 // This fixes #397, apparently if the height is 1, it can't render the CPU bars...
                 let cpu_height = {
-                    let c = (actual_cpu_data_len / 4) as u16
-                        + (if actual_cpu_data_len % 4 == 0 { 0 } else { 1 });
+                    let c =
+                        (actual_cpu_data_len / 4) as u16 + u16::from(actual_cpu_data_len % 4 != 0);
 
                     if c <= 1 {
                         1
@@ -541,12 +422,36 @@ impl Painter {
                     }
                 };
 
+                let mut mem_rows = 1;
+
+                if app_state.converted_data.swap_labels.is_some() {
+                    mem_rows += 1; // add row for swap
+                }
+
+                #[cfg(feature = "zfs")]
+                {
+                    if app_state.converted_data.arc_labels.is_some() {
+                        mem_rows += 1; // add row for arc
+                    }
+                }
+
+                #[cfg(feature = "gpu")]
+                {
+                    if let Some(gpu_data) = &app_state.converted_data.gpu_data {
+                        mem_rows += gpu_data.len() as u16; // add row(s) for gpu
+                    }
+                }
+
+                if mem_rows == 1 {
+                    mem_rows += 1; // need at least 2 rows for RX and TX
+                }
+
                 let vertical_chunks = Layout::default()
                     .direction(Direction::Vertical)
                     .margin(0)
                     .constraints([
                         Constraint::Length(cpu_height),
-                        Constraint::Length(2),
+                        Constraint::Length(mem_rows),
                         Constraint::Length(2),
                         Constraint::Min(5),
                     ])
@@ -573,13 +478,9 @@ impl Painter {
                     later_widget_id = Some(widget_id);
                     if vertical_chunks[3].width >= 2 {
                         match basic_table_widget_state.currently_displayed_widget_type {
-                            Disk => self.draw_disk_table(
-                                f,
-                                app_state,
-                                vertical_chunks[3],
-                                false,
-                                widget_id,
-                            ),
+                            Disk => {
+                                self.draw_disk_table(f, app_state, vertical_chunks[3], widget_id)
+                            }
                             Proc | ProcSort => {
                                 let wid = widget_id
                                     - match basic_table_widget_state.currently_displayed_widget_type
@@ -588,7 +489,7 @@ impl Painter {
                                         ProcSort => 2,
                                         _ => 0,
                                     };
-                                self.draw_process_features(
+                                self.draw_process_widget(
                                     f,
                                     app_state,
                                     vertical_chunks[3],
@@ -596,13 +497,9 @@ impl Painter {
                                     wid,
                                 );
                             }
-                            Temp => self.draw_temp_table(
-                                f,
-                                app_state,
-                                vertical_chunks[3],
-                                false,
-                                widget_id,
-                            ),
+                            Temp => {
+                                self.draw_temp_table(f, app_state, vertical_chunks[3], widget_id)
+                            }
                             Battery => self.draw_battery_display(
                                 f,
                                 app_state,
@@ -625,11 +522,160 @@ impl Painter {
                 }
 
                 if self.derived_widget_draw_locs.is_empty() || app_state.is_force_redraw {
-                    let draw_locs = Layout::default()
-                        .margin(0)
-                        .constraints(self.row_constraints.as_ref())
-                        .direction(Direction::Vertical)
-                        .split(terminal_size);
+                    fn get_constraints(
+                        direction: Direction, constraints: &[LayoutConstraint], area: Rect,
+                    ) -> Vec<Rect> {
+                        // Order of operations:
+                        // - Ratios first + canvas-handled (which is just zero)
+                        // - Then any flex-grows to take up remaining space; divide amongst remaining
+                        //   hand out any remaining space
+
+                        #[derive(Debug, Default, Clone, Copy)]
+                        struct Size {
+                            width: u16,
+                            height: u16,
+                        }
+
+                        impl Size {
+                            fn shrink_width(&mut self, amount: u16) {
+                                self.width -= amount;
+                            }
+
+                            fn shrink_height(&mut self, amount: u16) {
+                                self.height -= amount;
+                            }
+                        }
+
+                        let mut bounds = Size {
+                            width: area.width,
+                            height: area.height,
+                        };
+                        let mut sizes = vec![Size::default(); constraints.len()];
+                        let mut grow = vec![];
+                        let mut num_non_ch = 0;
+
+                        for (itx, (constraint, size)) in
+                            constraints.iter().zip(sizes.iter_mut()).enumerate()
+                        {
+                            match constraint {
+                                LayoutConstraint::Ratio(a, b) => {
+                                    match direction {
+                                        Direction::Horizontal => {
+                                            let amount =
+                                                (((area.width as u32) * (*a)) / (*b)) as u16;
+                                            bounds.shrink_width(amount);
+                                            size.width = amount;
+                                            size.height = area.height;
+                                        }
+                                        Direction::Vertical => {
+                                            let amount =
+                                                (((area.height as u32) * (*a)) / (*b)) as u16;
+                                            bounds.shrink_height(amount);
+                                            size.width = area.width;
+                                            size.height = amount;
+                                        }
+                                    }
+                                    num_non_ch += 1;
+                                }
+                                LayoutConstraint::Grow => {
+                                    // Mark it as grow in the vector and handle in second pass.
+                                    grow.push(itx);
+                                    num_non_ch += 1;
+                                }
+                                LayoutConstraint::CanvasHandled => {
+                                    // Do nothing in this case. It's already 0.
+                                }
+                            }
+                        }
+
+                        if !grow.is_empty() {
+                            match direction {
+                                Direction::Horizontal => {
+                                    let width = bounds.width / grow.len() as u16;
+                                    bounds.shrink_width(width * grow.len() as u16);
+                                    for g in grow {
+                                        sizes[g] = Size {
+                                            width,
+                                            height: area.height,
+                                        };
+                                    }
+                                }
+                                Direction::Vertical => {
+                                    let height = bounds.height / grow.len() as u16;
+                                    bounds.shrink_height(height * grow.len() as u16);
+                                    for g in grow {
+                                        sizes[g] = Size {
+                                            width: area.width,
+                                            height,
+                                        };
+                                    }
+                                }
+                            }
+                        }
+
+                        if num_non_ch > 0 {
+                            match direction {
+                                Direction::Horizontal => {
+                                    let per_item = bounds.width / num_non_ch;
+                                    let mut remaining_width = bounds.width % num_non_ch;
+                                    for (size, constraint) in sizes.iter_mut().zip(constraints) {
+                                        match constraint {
+                                            LayoutConstraint::CanvasHandled => {}
+                                            LayoutConstraint::Grow
+                                            | LayoutConstraint::Ratio(_, _) => {
+                                                if remaining_width > 0 {
+                                                    size.width += per_item + 1;
+                                                    remaining_width -= 1;
+                                                } else {
+                                                    size.width += per_item;
+                                                }
+                                            }
+                                        }
+                                    }
+                                }
+                                Direction::Vertical => {
+                                    let per_item = bounds.height / num_non_ch;
+                                    let mut remaining_height = bounds.height % num_non_ch;
+                                    for (size, constraint) in sizes.iter_mut().zip(constraints) {
+                                        match constraint {
+                                            LayoutConstraint::CanvasHandled => {}
+                                            LayoutConstraint::Grow
+                                            | LayoutConstraint::Ratio(_, _) => {
+                                                if remaining_height > 0 {
+                                                    size.height += per_item + 1;
+                                                    remaining_height -= 1;
+                                                } else {
+                                                    size.height += per_item;
+                                                }
+                                            }
+                                        }
+                                    }
+                                }
+                            }
+                        }
+
+                        let mut curr_x = area.x;
+                        let mut curr_y = area.y;
+                        sizes
+                            .into_iter()
+                            .map(|size| {
+                                let rect = Rect::new(curr_x, curr_y, size.width, size.height);
+                                match direction {
+                                    Direction::Horizontal => {
+                                        curr_x += size.width;
+                                    }
+                                    Direction::Vertical => {
+                                        curr_y += size.height;
+                                    }
+                                }
+
+                                rect
+                            })
+                            .collect()
+                    }
+
+                    let draw_locs =
+                        get_constraints(Direction::Vertical, &self.row_constraints, terminal_size);
 
                     self.derived_widget_draw_locs = izip!(
                         draw_locs,
@@ -647,31 +693,28 @@ impl Painter {
                             cols,
                         )| {
                             izip!(
-                                Layout::default()
-                                    .constraints(col_constraint.as_ref())
-                                    .direction(Direction::Horizontal)
-                                    .split(draw_loc)
-                                    .into_iter(),
+                                get_constraints(Direction::Horizontal, col_constraint, draw_loc),
                                 col_row_constraint,
                                 row_constraint_vec,
                                 &cols.children
                             )
                             .map(|(split_loc, constraint, col_constraint_vec, col_rows)| {
                                 izip!(
-                                    Layout::default()
-                                        .constraints(constraint.as_ref())
-                                        .direction(Direction::Vertical)
-                                        .split(split_loc)
-                                        .into_iter(),
+                                    get_constraints(
+                                        Direction::Vertical,
+                                        constraint.as_slice(),
+                                        split_loc
+                                    ),
                                     col_constraint_vec,
                                     &col_rows.children
                                 )
                                 .map(|(draw_loc, col_row_constraint_vec, widgets)| {
                                     // Note that col_row_constraint_vec CONTAINS the widget constraints
-                                    let widget_draw_locs = Layout::default()
-                                        .constraints(col_row_constraint_vec.as_ref())
-                                        .direction(Direction::Horizontal)
-                                        .split(draw_loc);
+                                    let widget_draw_locs = get_constraints(
+                                        Direction::Horizontal,
+                                        col_row_constraint_vec.as_slice(),
+                                        draw_loc,
+                                    );
 
                                     // Side effect, draw here.
                                     self.draw_widgets_with_constraints(
@@ -733,13 +776,9 @@ impl Painter {
                     Cpu => self.draw_cpu(f, app_state, *widget_draw_loc, widget.widget_id),
                     Mem => self.draw_memory_graph(f, app_state, *widget_draw_loc, widget.widget_id),
                     Net => self.draw_network(f, app_state, *widget_draw_loc, widget.widget_id),
-                    Temp => {
-                        self.draw_temp_table(f, app_state, *widget_draw_loc, true, widget.widget_id)
-                    }
-                    Disk => {
-                        self.draw_disk_table(f, app_state, *widget_draw_loc, true, widget.widget_id)
-                    }
-                    Proc => self.draw_process_features(
+                    Temp => self.draw_temp_table(f, app_state, *widget_draw_loc, widget.widget_id),
+                    Disk => self.draw_disk_table(f, app_state, *widget_draw_loc, widget.widget_id),
+                    Proc => self.draw_process_widget(
                         f,
                         app_state,
                         *widget_draw_loc,
