@@ -1,9 +1,3 @@
-use crate::{
-    app::App,
-    canvas::{drawing_utils::calculate_basic_use_bars, Painter},
-    constants::*,
-};
-
 use tui::{
     backend::Backend,
     layout::{Constraint, Direction, Layout, Rect},
@@ -13,15 +7,14 @@ use tui::{
 };
 use unicode_segmentation::UnicodeSegmentation;
 
-pub trait BatteryDisplayWidget {
-    fn draw_battery_display<B: Backend>(
-        &self, f: &mut Frame<'_, B>, app_state: &mut App, draw_loc: Rect, draw_border: bool,
-        widget_id: u64,
-    );
-}
+use crate::{
+    app::App,
+    canvas::{drawing_utils::calculate_basic_use_bars, Painter},
+    constants::*,
+};
 
-impl BatteryDisplayWidget for Painter {
-    fn draw_battery_display<B: Backend>(
+impl Painter {
+    pub fn draw_battery_display<B: Backend>(
         &self, f: &mut Frame<'_, B>, app_state: &mut App, draw_loc: Rect, draw_border: bool,
         widget_id: u64,
     ) {
@@ -44,7 +37,7 @@ impl BatteryDisplayWidget for Painter {
             let title = if app_state.is_expanded {
                 const TITLE_BASE: &str = " Battery ── Esc to go back ";
                 Spans::from(vec![
-                    Span::styled(" Battery ".to_string(), self.colours.widget_title_style),
+                    Span::styled(" Battery ", self.colours.widget_title_style),
                     Span::styled(
                         format!(
                             "─{}─ Esc to go back ",
@@ -56,10 +49,7 @@ impl BatteryDisplayWidget for Painter {
                     ),
                 ])
             } else {
-                Spans::from(Span::styled(
-                    " Battery ".to_string(),
-                    self.colours.widget_title_style,
-                ))
+                Spans::from(Span::styled(" Battery ", self.colours.widget_title_style))
             };
 
             let battery_block = if draw_border {
@@ -69,14 +59,14 @@ impl BatteryDisplayWidget for Painter {
                     .border_style(border_style)
             } else if is_on_widget {
                 Block::default()
-                    .borders(*SIDE_BORDERS)
+                    .borders(SIDE_BORDERS)
                     .border_style(self.colours.highlighted_border_style)
             } else {
                 Block::default().borders(Borders::NONE)
             };
 
             let battery_names = app_state
-                .canvas_data
+                .converted_data
                 .battery_data
                 .iter()
                 .map(|battery| &battery.battery_name)
@@ -108,12 +98,12 @@ impl BatteryDisplayWidget for Painter {
 
             let margined_draw_loc = Layout::default()
                 .constraints([Constraint::Percentage(100)])
-                .horizontal_margin(if is_on_widget || draw_border { 0 } else { 1 })
+                .horizontal_margin(u16::from(!(is_on_widget || draw_border)))
                 .direction(Direction::Horizontal)
                 .split(draw_loc)[0];
 
             if let Some(battery_details) = app_state
-                .canvas_data
+                .converted_data
                 .battery_data
                 .get(battery_widget_state.currently_selected_battery_index)
             {
@@ -165,7 +155,7 @@ impl BatteryDisplayWidget for Painter {
                     margined_draw_loc,
                 );
             } else {
-                let mut contents = vec![Spans::default(); table_gap as usize];
+                let mut contents = vec![Spans::default(); table_gap.into()];
 
                 contents.push(Spans::from(Span::styled(
                     "No data found for this battery",
