@@ -1,12 +1,10 @@
-use crate::options::ConfigColours;
 use once_cell::sync::Lazy;
 
+use crate::options::ConfigColours;
+
 // Default widget ID
 pub const DEFAULT_WIDGET_ID: u64 = 56709;
 
-// How long to store data.
-pub const STALE_MAX_MILLISECONDS: u64 = 600 * 1000; // Keep 10 minutes of data.
-
 // How much data is SHOWN
 pub const DEFAULT_TIME_MILLISECONDS: u64 = 60 * 1000; // Defaults to 1 min.
 pub const STALE_MIN_MILLISECONDS: u64 = 30 * 1000; // Lowest is 30 seconds
@@ -31,190 +29,240 @@ pub static DEFAULT_HEADER_STYLE: Lazy<tui::style::Style> =
 
 // Colour profiles
 pub static DEFAULT_LIGHT_MODE_COLOUR_PALETTE: Lazy<ConfigColours> = Lazy::new(|| ConfigColours {
-    text_color: Some("black".to_string()),
-    border_color: Some("black".to_string()),
-    table_header_color: Some("black".to_string()),
-    widget_title_color: Some("black".to_string()),
-    selected_text_color: Some("white".to_string()),
-    graph_color: Some("black".to_string()),
-    disabled_text_color: Some("gray".to_string()),
-    ram_color: Some("blue".to_string()),
-    swap_color: Some("red".to_string()),
-    rx_color: Some("blue".to_string()),
-    tx_color: Some("red".to_string()),
-    rx_total_color: Some("LightBlue".to_string()),
-    tx_total_color: Some("LightRed".to_string()),
+    text_color: Some("black".into()),
+    border_color: Some("black".into()),
+    table_header_color: Some("black".into()),
+    widget_title_color: Some("black".into()),
+    selected_text_color: Some("white".into()),
+    graph_color: Some("black".into()),
+    disabled_text_color: Some("gray".into()),
+    ram_color: Some("blue".into()),
+    swap_color: Some("red".into()),
+    arc_color: Some("LightBlue".into()),
+    gpu_core_colors: Some(vec![
+        "LightGreen".into(),
+        "LightCyan".into(),
+        "LightRed".into(),
+        "Cyan".into(),
+        "Green".into(),
+        "Blue".into(),
+        "Red".into(),
+    ]),
+    rx_color: Some("blue".into()),
+    tx_color: Some("red".into()),
+    rx_total_color: Some("LightBlue".into()),
+    tx_total_color: Some("LightRed".into()),
     cpu_core_colors: Some(vec![
-        "LightMagenta".to_string(),
-        "LightBlue".to_string(),
-        "LightRed".to_string(),
-        "Cyan".to_string(),
-        "Green".to_string(),
-        "Blue".to_string(),
-        "Red".to_string(),
+        "LightMagenta".into(),
+        "LightBlue".into(),
+        "LightRed".into(),
+        "Cyan".into(),
+        "Green".into(),
+        "Blue".into(),
+        "Red".into(),
     ]),
     ..ConfigColours::default()
 });
 
 pub static GRUVBOX_COLOUR_PALETTE: Lazy<ConfigColours> = Lazy::new(|| ConfigColours {
-    table_header_color: Some("#83a598".to_string()),
-    all_cpu_color: Some("#8ec07c".to_string()),
-    avg_cpu_color: Some("#fb4934".to_string()),
+    table_header_color: Some("#83a598".into()),
+    all_cpu_color: Some("#8ec07c".into()),
+    avg_cpu_color: Some("#fb4934".into()),
     cpu_core_colors: Some(vec![
-        "#cc241d".to_string(),
-        "#98971a".to_string(),
-        "#d79921".to_string(),
-        "#458588".to_string(),
-        "#b16286".to_string(),
-        "#689d6a".to_string(),
-        "#fe8019".to_string(),
-        "#b8bb26".to_string(),
-        "#fabd2f".to_string(),
-        "#83a598".to_string(),
-        "#d3869b".to_string(),
-        "#d65d0e".to_string(),
-        "#9d0006".to_string(),
-        "#79740e".to_string(),
-        "#b57614".to_string(),
-        "#076678".to_string(),
-        "#8f3f71".to_string(),
-        "#427b58".to_string(),
-        "#d65d03".to_string(),
-        "#af3a03".to_string(),
+        "#cc241d".into(),
+        "#98971a".into(),
+        "#d79921".into(),
+        "#458588".into(),
+        "#b16286".into(),
+        "#689d6a".into(),
+        "#fe8019".into(),
+        "#b8bb26".into(),
+        "#fabd2f".into(),
+        "#83a598".into(),
+        "#d3869b".into(),
+        "#d65d0e".into(),
+        "#9d0006".into(),
+        "#79740e".into(),
+        "#b57614".into(),
+        "#076678".into(),
+        "#8f3f71".into(),
+        "#427b58".into(),
+        "#d65d03".into(),
+        "#af3a03".into(),
     ]),
-    ram_color: Some("#8ec07c".to_string()),
-    swap_color: Some("#fabd2f".to_string()),
-    rx_color: Some("#8ec07c".to_string()),
-    tx_color: Some("#fabd2f".to_string()),
-    rx_total_color: Some("#689d6a".to_string()),
-    tx_total_color: Some("#d79921".to_string()),
-    border_color: Some("#ebdbb2".to_string()),
-    highlighted_border_color: Some("#fe8019".to_string()),
-    disabled_text_color: Some("#665c54".to_string()),
-    text_color: Some("#ebdbb2".to_string()),
-    selected_text_color: Some("#1d2021".to_string()),
-    selected_bg_color: Some("#ebdbb2".to_string()),
-    widget_title_color: Some("#ebdbb2".to_string()),
-    graph_color: Some("#ebdbb2".to_string()),
-    high_battery_color: Some("#98971a".to_string()),
-    medium_battery_color: Some("#fabd2f".to_string()),
-    low_battery_color: Some("#fb4934".to_string()),
+    ram_color: Some("#8ec07c".into()),
+    swap_color: Some("#fabd2f".into()),
+    arc_color: Some("#689d6a".into()),
+    gpu_core_colors: Some(vec![
+        "#d79921".into(),
+        "#458588".into(),
+        "#b16286".into(),
+        "#fe8019".into(),
+        "#b8bb26".into(),
+        "#cc241d".into(),
+        "#98971a".into(),
+    ]),
+    rx_color: Some("#8ec07c".into()),
+    tx_color: Some("#fabd2f".into()),
+    rx_total_color: Some("#689d6a".into()),
+    tx_total_color: Some("#d79921".into()),
+    border_color: Some("#ebdbb2".into()),
+    highlighted_border_color: Some("#fe8019".into()),
+    disabled_text_color: Some("#665c54".into()),
+    text_color: Some("#ebdbb2".into()),
+    selected_text_color: Some("#1d2021".into()),
+    selected_bg_color: Some("#ebdbb2".into()),
+    widget_title_color: Some("#ebdbb2".into()),
+    graph_color: Some("#ebdbb2".into()),
+    high_battery_color: Some("#98971a".into()),
+    medium_battery_color: Some("#fabd2f".into()),
+    low_battery_color: Some("#fb4934".into()),
 });
 
 pub static GRUVBOX_LIGHT_COLOUR_PALETTE: Lazy<ConfigColours> = Lazy::new(|| ConfigColours {
-    table_header_color: Some("#076678".to_string()),
-    all_cpu_color: Some("#8ec07c".to_string()),
-    avg_cpu_color: Some("#fb4934".to_string()),
+    table_header_color: Some("#076678".into()),
+    all_cpu_color: Some("#8ec07c".into()),
+    avg_cpu_color: Some("#fb4934".into()),
     cpu_core_colors: Some(vec![
-        "#cc241d".to_string(),
-        "#98971a".to_string(),
-        "#d79921".to_string(),
-        "#458588".to_string(),
-        "#b16286".to_string(),
-        "#689d6a".to_string(),
-        "#fe8019".to_string(),
-        "#b8bb26".to_string(),
-        "#fabd2f".to_string(),
-        "#83a598".to_string(),
-        "#d3869b".to_string(),
-        "#d65d0e".to_string(),
-        "#9d0006".to_string(),
-        "#79740e".to_string(),
-        "#b57614".to_string(),
-        "#076678".to_string(),
-        "#8f3f71".to_string(),
-        "#427b58".to_string(),
-        "#d65d03".to_string(),
-        "#af3a03".to_string(),
+        "#cc241d".into(),
+        "#98971a".into(),
+        "#d79921".into(),
+        "#458588".into(),
+        "#b16286".into(),
+        "#689d6a".into(),
+        "#fe8019".into(),
+        "#b8bb26".into(),
+        "#fabd2f".into(),
+        "#83a598".into(),
+        "#d3869b".into(),
+        "#d65d0e".into(),
+        "#9d0006".into(),
+        "#79740e".into(),
+        "#b57614".into(),
+        "#076678".into(),
+        "#8f3f71".into(),
+        "#427b58".into(),
+        "#d65d03".into(),
+        "#af3a03".into(),
     ]),
-    ram_color: Some("#427b58".to_string()),
-    swap_color: Some("#cc241d".to_string()),
-    rx_color: Some("#427b58".to_string()),
-    tx_color: Some("#cc241d".to_string()),
-    rx_total_color: Some("#689d6a".to_string()),
-    tx_total_color: Some("#9d0006".to_string()),
-    border_color: Some("#3c3836".to_string()),
-    highlighted_border_color: Some("#af3a03".to_string()),
-    disabled_text_color: Some("#d5c4a1".to_string()),
-    text_color: Some("#3c3836".to_string()),
-    selected_text_color: Some("#ebdbb2".to_string()),
-    selected_bg_color: Some("#3c3836".to_string()),
-    widget_title_color: Some("#3c3836".to_string()),
-    graph_color: Some("#3c3836".to_string()),
-    high_battery_color: Some("#98971a".to_string()),
-    medium_battery_color: Some("#d79921".to_string()),
-    low_battery_color: Some("#cc241d".to_string()),
+    ram_color: Some("#427b58".into()),
+    swap_color: Some("#cc241d".into()),
+    arc_color: Some("#689d6a".into()),
+    gpu_core_colors: Some(vec![
+        "#9d0006".into(),
+        "#98971a".into(),
+        "#d79921".into(),
+        "#458588".into(),
+        "#b16286".into(),
+        "#fe8019".into(),
+        "#b8bb26".into(),
+    ]),
+    rx_color: Some("#427b58".into()),
+    tx_color: Some("#cc241d".into()),
+    rx_total_color: Some("#689d6a".into()),
+    tx_total_color: Some("#9d0006".into()),
+    border_color: Some("#3c3836".into()),
+    highlighted_border_color: Some("#af3a03".into()),
+    disabled_text_color: Some("#d5c4a1".into()),
+    text_color: Some("#3c3836".into()),
+    selected_text_color: Some("#ebdbb2".into()),
+    selected_bg_color: Some("#3c3836".into()),
+    widget_title_color: Some("#3c3836".into()),
+    graph_color: Some("#3c3836".into()),
+    high_battery_color: Some("#98971a".into()),
+    medium_battery_color: Some("#d79921".into()),
+    low_battery_color: Some("#cc241d".into()),
 });
 
 pub static NORD_COLOUR_PALETTE: Lazy<ConfigColours> = Lazy::new(|| ConfigColours {
-    table_header_color: Some("#81a1c1".to_string()),
-    all_cpu_color: Some("#88c0d0".to_string()),
-    avg_cpu_color: Some("#8fbcbb".to_string()),
+    table_header_color: Some("#81a1c1".into()),
+    all_cpu_color: Some("#88c0d0".into()),
+    avg_cpu_color: Some("#8fbcbb".into()),
     cpu_core_colors: Some(vec![
-        "#5e81ac".to_string(),
-        "#81a1c1".to_string(),
-        "#d8dee9".to_string(),
-        "#b48ead".to_string(),
-        "#a3be8c".to_string(),
-        "#ebcb8b".to_string(),
-        "#d08770".to_string(),
-        "#bf616a".to_string(),
+        "#5e81ac".into(),
+        "#81a1c1".into(),
+        "#d8dee9".into(),
+        "#b48ead".into(),
+        "#a3be8c".into(),
+        "#ebcb8b".into(),
+        "#d08770".into(),
+        "#bf616a".into(),
     ]),
-    ram_color: Some("#88c0d0".to_string()),
-    swap_color: Some("#d08770".to_string()),
-    rx_color: Some("#88c0d0".to_string()),
-    tx_color: Some("#d08770".to_string()),
-    rx_total_color: Some("#5e81ac".to_string()),
-    tx_total_color: Some("#8fbcbb".to_string()),
-    border_color: Some("#88c0d0".to_string()),
-    highlighted_border_color: Some("#5e81ac".to_string()),
-    disabled_text_color: Some("#4c566a".to_string()),
-    text_color: Some("#e5e9f0".to_string()),
-    selected_text_color: Some("#2e3440".to_string()),
-    selected_bg_color: Some("#88c0d0".to_string()),
-    widget_title_color: Some("#e5e9f0".to_string()),
-    graph_color: Some("#e5e9f0".to_string()),
-    high_battery_color: Some("#a3be8c".to_string()),
-    medium_battery_color: Some("#ebcb8b".to_string()),
-    low_battery_color: Some("#bf616a".to_string()),
+    ram_color: Some("#88c0d0".into()),
+    swap_color: Some("#d08770".into()),
+    arc_color: Some("#5e81ac".into()),
+    gpu_core_colors: Some(vec![
+        "#8fbcbb".into(),
+        "#81a1c1".into(),
+        "#d8dee9".into(),
+        "#b48ead".into(),
+        "#a3be8c".into(),
+        "#ebcb8b".into(),
+        "#bf616a".into(),
+    ]),
+    rx_color: Some("#88c0d0".into()),
+    tx_color: Some("#d08770".into()),
+    rx_total_color: Some("#5e81ac".into()),
+    tx_total_color: Some("#8fbcbb".into()),
+    border_color: Some("#88c0d0".into()),
+    highlighted_border_color: Some("#5e81ac".into()),
+    disabled_text_color: Some("#4c566a".into()),
+    text_color: Some("#e5e9f0".into()),
+    selected_text_color: Some("#2e3440".into()),
+    selected_bg_color: Some("#88c0d0".into()),
+    widget_title_color: Some("#e5e9f0".into()),
+    graph_color: Some("#e5e9f0".into()),
+    high_battery_color: Some("#a3be8c".into()),
+    medium_battery_color: Some("#ebcb8b".into()),
+    low_battery_color: Some("#bf616a".into()),
 });
 
 pub static NORD_LIGHT_COLOUR_PALETTE: Lazy<ConfigColours> = Lazy::new(|| ConfigColours {
-    table_header_color: Some("#5e81ac".to_string()),
-    all_cpu_color: Some("#81a1c1".to_string()),
-    avg_cpu_color: Some("#8fbcbb".to_string()),
+    table_header_color: Some("#5e81ac".into()),
+    all_cpu_color: Some("#81a1c1".into()),
+    avg_cpu_color: Some("#8fbcbb".into()),
     cpu_core_colors: Some(vec![
-        "#5e81ac".to_string(),
-        "#88c0d0".to_string(),
-        "#4c566a".to_string(),
-        "#b48ead".to_string(),
-        "#a3be8c".to_string(),
-        "#ebcb8b".to_string(),
-        "#d08770".to_string(),
-        "#bf616a".to_string(),
+        "#5e81ac".into(),
+        "#88c0d0".into(),
+        "#4c566a".into(),
+        "#b48ead".into(),
+        "#a3be8c".into(),
+        "#ebcb8b".into(),
+        "#d08770".into(),
+        "#bf616a".into(),
     ]),
-    ram_color: Some("#81a1c1".to_string()),
-    swap_color: Some("#d08770".to_string()),
-    rx_color: Some("#81a1c1".to_string()),
-    tx_color: Some("#d08770".to_string()),
-    rx_total_color: Some("#5e81ac".to_string()),
-    tx_total_color: Some("#8fbcbb".to_string()),
-    border_color: Some("#2e3440".to_string()),
-    highlighted_border_color: Some("#5e81ac".to_string()),
-    disabled_text_color: Some("#d8dee9".to_string()),
-    text_color: Some("#2e3440".to_string()),
-    selected_text_color: Some("#f5f5f5".to_string()),
-    selected_bg_color: Some("#5e81ac".to_string()),
-    widget_title_color: Some("#2e3440".to_string()),
-    graph_color: Some("#2e3440".to_string()),
-    high_battery_color: Some("#a3be8c".to_string()),
-    medium_battery_color: Some("#ebcb8b".to_string()),
-    low_battery_color: Some("#bf616a".to_string()),
+    ram_color: Some("#81a1c1".into()),
+    swap_color: Some("#d08770".into()),
+    arc_color: Some("#5e81ac".into()),
+    gpu_core_colors: Some(vec![
+        "#8fbcbb".into(),
+        "#88c0d0".into(),
+        "#4c566a".into(),
+        "#b48ead".into(),
+        "#a3be8c".into(),
+        "#ebcb8b".into(),
+        "#bf616a".into(),
+    ]),
+    rx_color: Some("#81a1c1".into()),
+    tx_color: Some("#d08770".into()),
+    rx_total_color: Some("#5e81ac".into()),
+    tx_total_color: Some("#8fbcbb".into()),
+    border_color: Some("#2e3440".into()),
+    highlighted_border_color: Some("#5e81ac".into()),
+    disabled_text_color: Some("#d8dee9".into()),
+    text_color: Some("#2e3440".into()),
+    selected_text_color: Some("#f5f5f5".into()),
+    selected_bg_color: Some("#5e81ac".into()),
+    widget_title_color: Some("#2e3440".into()),
+    graph_color: Some("#2e3440".into()),
+    high_battery_color: Some("#a3be8c".into()),
+    medium_battery_color: Some("#ebcb8b".into()),
+    low_battery_color: Some("#bf616a".into()),
 });
 
 // Help text
 pub const HELP_CONTENTS_TEXT: [&str; 8] = [
-    "Press the corresponding numbers to jump to the section, or scroll:",
+    "Either scroll or press the number key to go to the corresponding help menu section:",
     "1 - General",
     "2 - CPU widget",
     "3 - Process widget",
@@ -226,7 +274,7 @@ pub const HELP_CONTENTS_TEXT: [&str; 8] = [
 
 // TODO [Help]: Search in help?
 // TODO [Help]: Move to using tables for easier formatting?
-pub const GENERAL_HELP_TEXT: [&str; 31] = [
+pub const GENERAL_HELP_TEXT: [&str; 32] = [
     "1 - General",
     "q, Ctrl-c        Quit",
     "Esc              Close dialog windows, search, widgets, or exit expanded mode",
@@ -256,6 +304,7 @@ pub const GENERAL_HELP_TEXT: [&str; 31] = [
     "-                Zoom out on chart (increase time range)",
     "=                Reset zoom",
     "PgUp, PgDown     Scroll up/down a table by a page",
+    "Ctrl-u, Ctrl-d   Scroll up/down a table by half a page",
     "Mouse scroll     Scroll through the tables or zoom in/out of charts by scrolling up/down",
     "Mouse click      Selects the clicked widget, table entry, dialog option, or tab",
 ];
@@ -443,6 +492,8 @@ pub const CONFIG_TEXT: &str = r##"# This is a default config file for bottom.  A
 #left_legend = false
 # Whether to set CPU% on a process to be based on the total CPU or just current usage.
 #current_usage = false
+# Whether to set CPU% on a process to be based on the total CPU or per-core CPU% (not divided by the number of cpus).
+#unnormalized_cpu = false
 # Whether to group processes with the same name together by default.
 #group_processes = false
 # Whether to make process searching case sensitive by default.
@@ -467,6 +518,8 @@ pub const CONFIG_TEXT: &str = r##"# This is a default config file for bottom.  A
 # Override layout default widget
 #default_widget_type = "proc"
 #default_widget_count = 1
+# Expand selected widget upon starting the app
+#expanded_on_startup = true
 # Use basic mode
 #basic = false
 # Use the old network legend style
@@ -495,6 +548,10 @@ pub const CONFIG_TEXT: &str = r##"# This is a default config file for bottom.  A
 #network_use_log = false
 # Hides advanced options to stop a process on Unix-like systems.
 #disable_advanced_kill = false
+# Shows GPU(s) memory
+#enable_gpu_memory = false
+# How much data is stored at once in terms of time.
+#retention = "10m"
 
 # These are all the components that support custom theming.  Note that colour support
 # will depend on terminal support.
@@ -512,6 +569,10 @@ pub const CONFIG_TEXT: &str = r##"# This is a default config file for bottom.  A
 #ram_color="LightMagenta"
 # Represents the colour SWAP will use in the memory legend and graph.
 #swap_color="LightYellow"
+# Represents the colour ARC will use in the memory legend and graph.
+#arc_color="LightCyan"
+# Represents the colour the GPU will use in the memory legend and graph.
+#gpu_core_colors=["LightGreen", "LightBlue", "LightRed", "Cyan", "Green", "Blue", "Red"]
 # Represents the colour rx will use in the network legend and graph.
 #rx_color="LightCyan"
 # Represents the colour tx will use in the network legend and graph.
