@@ -1,3 +1,10 @@
+//! A customizable cross-platform graphical process/system monitor for the terminal.
+//! Supports Linux, macOS, and Windows. Inspired by gtop, gotop, and htop.
+//!
+//! **Note:** The following documentation is primarily intended for people to refer to for development purposes rather
+//! than the actual usage of the application. If you are instead looking for documentation regarding the *usage* of
+//! bottom, refer to [here](https://clementtsang.github.io/bottom/stable/).
+
 #![warn(rust_2018_idioms)]
 #[allow(unused_imports)]
 #[cfg(feature = "log")]
@@ -9,29 +16,34 @@ extern crate log;
 use std::{
     boxed::Box,
     fs,
-    io::{stdout, Write},
+    io::{stderr, stdout, Write},
     panic::PanicInfo,
     path::PathBuf,
-    sync::Arc,
-    sync::Condvar,
     sync::Mutex,
-    thread,
+    sync::{
+        mpsc::{Receiver, Sender},
+        Arc, Condvar,
+    },
+    thread::{self, JoinHandle},
     time::{Duration, Instant},
 };
 
-use crossterm::{
-    event::{poll, read, DisableMouseCapture, Event, KeyCode, KeyEvent, KeyModifiers, MouseEvent},
-    execute,
-    style::Print,
-    terminal::{disable_raw_mode, LeaveAlternateScreen},
-};
-
 use app::{
-    data_harvester::{self, processes::ProcessSorting},
+    data_harvester,
+    frozen_state::FrozenState,
     layout_manager::{UsedWidgets, WidgetDirection},
     App,
 };
 use constants::*;
+use crossterm::{
+    event::{
+        poll, read, DisableBracketedPaste, DisableMouseCapture, Event, KeyCode, KeyEvent,
+        KeyModifiers, MouseEvent, MouseEventKind,
+    },
+    execute,
+    style::Print,
+    terminal::{disable_raw_mode, LeaveAlternateScreen},
+};
 use data_conversion::*;
 use options::*;
 use utils::error;
@@ -44,10 +56,12 @@ pub mod utils {
 }
 pub mod canvas;
 pub mod clap;
+pub mod components;
 pub mod constants;
 pub mod data_conversion;
 pub mod options;
 pub mod units;
+pub mod widgets;
 
 #[cfg(target_family = "windows")]
 pub type Pid = usize;
@@ -56,9 +70,11 @@ pub type Pid = usize;
 pub type Pid = libc::pid_t;
 
 #[derive(Debug)]
-pub enum BottomEvent<I, J> {
-    KeyInput(I),
-    MouseInput(J),
+pub enum BottomEvent {
+    Resize,
+    KeyInput(KeyEvent),
+    MouseInput(MouseEvent),
+    PasteEvent(String),
     Update(Box<data_harvester::Data>),
     Clean,
 }
@@ -72,10 +88,11 @@ pub enum ThreadControlEvent {
 }
 
 pub fn handle_mouse_event(event: MouseEvent, app: &mut App) {
-    match event {
-        MouseEvent::ScrollUp(_x, _y, _modifiers) => app.handle_scroll_up(),
-        MouseEvent::ScrollDown(_x, _y, _modifiers) => app.handle_scroll_down(),
-        MouseEvent::Down(button, x, y, _modifiers) => {
+    match event.kind {
+        MouseEventKind::ScrollUp => app.handle_scroll_up(),
+        MouseEventKind::ScrollDown => app.handle_scroll_down(),
+        MouseEventKind::Down(button) => {
+            let (x, y) = (event.column, event.row);
             if !app.app_config_fields.disable_click {
                 match button {
                     crossterm::event::MouseButton::Left => {
@@ -92,7 +109,7 @@ pub fn handle_mouse_event(event: MouseEvent, app: &mut App) {
 }
 
 pub fn handle_key_event_or_break(
-    event: KeyEvent, app: &mut App, reset_sender: &std::sync::mpsc::Sender<ThreadControlEvent>,
+    event: KeyEvent, app: &mut App, reset_sender: &Sender<ThreadControlEvent>,
 ) -> bool {
     // debug!("KeyEvent: {:?}", event);
 
@@ -123,7 +140,7 @@ pub fn handle_key_event_or_break(
             KeyCode::F(2) => app.toggle_search_whole_word(),
             KeyCode::F(3) => app.toggle_search_regex(),
             KeyCode::F(5) => app.toggle_tree_mode(),
-            KeyCode::F(6) => app.toggle_sort(),
+            KeyCode::F(6) => app.toggle_sort_menu(),
             KeyCode::F(9) => app.start_killing_process(),
             KeyCode::PageDown => app.on_page_down(),
             KeyCode::PageUp => app.on_page_up(),
@@ -158,9 +175,11 @@ pub fn handle_key_event_or_break(
                 }
                 KeyCode::Char('a') => app.skip_cursor_beginning(),
                 KeyCode::Char('e') => app.skip_cursor_end(),
-                KeyCode::Char('u') => app.clear_search(),
+                KeyCode::Char('u') if app.is_in_search_widget() => app.clear_search(),
                 KeyCode::Char('w') => app.clear_previous_word(),
                 KeyCode::Char('h') => app.on_backspace(),
+                KeyCode::Char('d') => app.scroll_half_page_down(),
+                KeyCode::Char('u') => app.scroll_half_page_up(),
                 // KeyCode::Char('j') => {}, // Move down
                 // KeyCode::Char('k') => {}, // Move up
                 // KeyCode::Char('h') => {}, // Move right
@@ -258,21 +277,31 @@ pub fn cleanup_terminal(
     disable_raw_mode()?;
     execute!(
         terminal.backend_mut(),
+        DisableBracketedPaste,
         DisableMouseCapture,
         LeaveAlternateScreen
     )?;
     terminal.show_cursor()?;
 
-    // if is_debug {
-    //     let mut tmp_dir = std::env::temp_dir();
-    //     tmp_dir.push("bottom_debug.log");
-    //     println!("Your debug file is located at {:?}", tmp_dir.as_os_str());
-    // }
-
     Ok(())
 }
 
-/// Based on https://github.com/Rigellute/spotify-tui/blob/master/src/main.rs
+/// Check and report to the user if the current environment is not a terminal.
+pub fn check_if_terminal() {
+    use crossterm::tty::IsTty;
+
+    if !stdout().is_tty() {
+        eprintln!(
+            "Warning: bottom is not being output to a terminal. Things might not work properly."
+        );
+        eprintln!("If you're stuck, press 'q' or 'Ctrl-c' to quit the program.");
+        stderr().flush().unwrap();
+        thread::sleep(Duration::from_secs(1));
+    }
+}
+
+/// A panic hook to properly restore the terminal in the case of a panic.
+/// Based on [spotify-tui's implementation](https://github.com/Rigellute/spotify-tui/blob/master/src/main.rs).
 pub fn panic_hook(panic_info: &PanicInfo<'_>) {
     let mut stdout = stdout();
 
@@ -287,7 +316,13 @@ pub fn panic_hook(panic_info: &PanicInfo<'_>) {
     let stacktrace: String = format!("{:?}", backtrace::Backtrace::new());
 
     disable_raw_mode().unwrap();
-    execute!(stdout, DisableMouseCapture, LeaveAlternateScreen).unwrap();
+    execute!(
+        stdout,
+        DisableBracketedPaste,
+        DisableMouseCapture,
+        LeaveAlternateScreen
+    )
+    .unwrap();
 
     // Print stack trace.  Must be done after!
     execute!(
@@ -302,304 +337,87 @@ pub fn panic_hook(panic_info: &PanicInfo<'_>) {
     .unwrap();
 }
 
-pub fn handle_force_redraws(app: &mut App) {
-    // Currently we use an Option... because we might want to future-proof this
-    // if we eventually get widget-specific redrawing!
-    if app.proc_state.force_update_all {
-        update_all_process_lists(app);
-        app.proc_state.force_update_all = false;
-    } else if let Some(widget_id) = app.proc_state.force_update {
-        update_final_process_list(app, widget_id);
-        app.proc_state.force_update = None;
+pub fn update_data(app: &mut App) {
+    let data_source = match &app.frozen_state {
+        FrozenState::NotFrozen => &app.data_collection,
+        FrozenState::Frozen(data) => data,
+    };
+
+    for proc in app.proc_state.widget_states.values_mut() {
+        if proc.force_update_data {
+            proc.ingest_data(data_source);
+            proc.force_update_data = false;
+        }
     }
 
+    // FIXME: Make this CPU force update less terrible.
     if app.cpu_state.force_update.is_some() {
-        convert_cpu_data_points(
-            &app.data_collection,
-            &mut app.canvas_data.cpu_data,
-            app.is_frozen,
-        );
-        app.canvas_data.load_avg_data = app.data_collection.load_avg_harvest;
+        app.converted_data.ingest_cpu_data(data_source);
+        app.converted_data.load_avg_data = data_source.load_avg_harvest;
+
         app.cpu_state.force_update = None;
     }
 
-    // FIXME: [OPT] Prefer reassignment over new vectors?
+    // FIXME: This is a bit of a temp hack to move data over.
+    {
+        let data = &app.converted_data.cpu_data;
+        for cpu in app.cpu_state.widget_states.values_mut() {
+            cpu.update_table(data);
+        }
+    }
+    {
+        let data = &app.converted_data.temp_data;
+        for temp in app.temp_state.widget_states.values_mut() {
+            if temp.force_update_data {
+                temp.ingest_data(data);
+                temp.force_update_data = false;
+            }
+        }
+    }
+    {
+        let data = &app.converted_data.disk_data;
+        for disk in app.disk_state.widget_states.values_mut() {
+            if disk.force_update_data {
+                disk.ingest_data(data);
+                disk.force_update_data = false;
+            }
+        }
+    }
+
+    // TODO: [OPT] Prefer reassignment over new vectors?
     if app.mem_state.force_update.is_some() {
-        app.canvas_data.mem_data = convert_mem_data_points(&app.data_collection, app.is_frozen);
-        app.canvas_data.swap_data = convert_swap_data_points(&app.data_collection, app.is_frozen);
+        app.converted_data.mem_data = convert_mem_data_points(data_source);
+        app.converted_data.swap_data = convert_swap_data_points(data_source);
+        #[cfg(feature = "zfs")]
+        {
+            app.converted_data.arc_data = convert_arc_data_points(data_source);
+        }
+
+        #[cfg(feature = "gpu")]
+        {
+            app.converted_data.gpu_data = convert_gpu_data(data_source);
+        }
         app.mem_state.force_update = None;
     }
 
     if app.net_state.force_update.is_some() {
         let (rx, tx) = get_rx_tx_data_points(
-            &app.data_collection,
-            app.is_frozen,
+            data_source,
             &app.app_config_fields.network_scale_type,
             &app.app_config_fields.network_unit_type,
             app.app_config_fields.network_use_binary_prefix,
         );
-        app.canvas_data.network_data_rx = rx;
-        app.canvas_data.network_data_tx = tx;
+        app.converted_data.network_data_rx = rx;
+        app.converted_data.network_data_tx = tx;
         app.net_state.force_update = None;
     }
 }
 
-#[allow(clippy::needless_collect)]
-pub fn update_all_process_lists(app: &mut App) {
-    // According to clippy, I can avoid a collect... but if I follow it,
-    // I end up conflicting with the borrow checker since app is used within the closure... hm.
-    if !app.is_frozen {
-        let widget_ids = app
-            .proc_state
-            .widget_states
-            .keys()
-            .cloned()
-            .collect::<Vec<_>>();
-
-        widget_ids.into_iter().for_each(|widget_id| {
-            update_final_process_list(app, widget_id);
-        });
-    }
-}
-
-fn update_final_process_list(app: &mut App, widget_id: u64) {
-    let process_states = app
-        .proc_state
-        .widget_states
-        .get(&widget_id)
-        .map(|process_state| {
-            (
-                process_state
-                    .process_search_state
-                    .search_state
-                    .is_invalid_or_blank_search(),
-                process_state.is_using_command,
-                process_state.is_grouped,
-                process_state.is_tree_mode,
-            )
-        });
-
-    if let Some((is_invalid_or_blank, is_using_command, is_grouped, is_tree)) = process_states {
-        if !app.is_frozen {
-            convert_process_data(
-                &app.data_collection,
-                &mut app.canvas_data.single_process_data,
-                #[cfg(target_family = "unix")]
-                &mut app.user_table,
-            );
-        }
-        let process_filter = app.get_process_filter(widget_id);
-        let filtered_process_data: Vec<ConvertedProcessData> = if is_tree {
-            app.canvas_data
-                .single_process_data
-                .iter()
-                .map(|(_pid, process)| {
-                    let mut process_clone = process.clone();
-                    if !is_invalid_or_blank {
-                        if let Some(process_filter) = process_filter {
-                            process_clone.is_disabled_entry =
-                                !process_filter.check(&process_clone, is_using_command);
-                        }
-                    }
-                    process_clone
-                })
-                .collect::<Vec<_>>()
-        } else {
-            app.canvas_data
-                .single_process_data
-                .iter()
-                .filter_map(|(_pid, process)| {
-                    if !is_invalid_or_blank {
-                        if let Some(process_filter) = process_filter {
-                            if process_filter.check(process, is_using_command) {
-                                Some(process)
-                            } else {
-                                None
-                            }
-                        } else {
-                            Some(process)
-                        }
-                    } else {
-                        Some(process)
-                    }
-                })
-                .cloned()
-                .collect::<Vec<_>>()
-        };
-
-        if let Some(proc_widget_state) = app.proc_state.get_mut_widget_state(widget_id) {
-            let mut finalized_process_data = if is_tree {
-                tree_process_data(
-                    &filtered_process_data,
-                    is_using_command,
-                    &proc_widget_state.process_sorting_type,
-                    proc_widget_state.is_process_sort_descending,
-                )
-            } else if is_grouped {
-                group_process_data(&filtered_process_data, is_using_command)
-            } else {
-                filtered_process_data
-            };
-
-            // Note tree mode is sorted well before this, as it's special.
-            if !is_tree {
-                sort_process_data(&mut finalized_process_data, proc_widget_state);
-            }
-
-            if proc_widget_state.scroll_state.current_scroll_position
-                >= finalized_process_data.len()
-            {
-                proc_widget_state.scroll_state.current_scroll_position =
-                    finalized_process_data.len().saturating_sub(1);
-                proc_widget_state.scroll_state.previous_scroll_position = 0;
-                proc_widget_state.scroll_state.scroll_direction = app::ScrollDirection::Down;
-            }
-
-            app.canvas_data.stringified_process_data_map.insert(
-                widget_id,
-                stringify_process_data(proc_widget_state, &finalized_process_data),
-            );
-            app.canvas_data
-                .finalized_process_data_map
-                .insert(widget_id, finalized_process_data);
-        }
-    }
-}
-
-fn sort_process_data(
-    to_sort_vec: &mut Vec<ConvertedProcessData>, proc_widget_state: &app::ProcWidgetState,
-) {
-    to_sort_vec.sort_by_cached_key(|c| c.name.to_lowercase());
-
-    match &proc_widget_state.process_sorting_type {
-        ProcessSorting::CpuPercent => {
-            to_sort_vec.sort_by(|a, b| {
-                utils::gen_util::get_ordering(
-                    a.cpu_percent_usage,
-                    b.cpu_percent_usage,
-                    proc_widget_state.is_process_sort_descending,
-                )
-            });
-        }
-        ProcessSorting::Mem => {
-            to_sort_vec.sort_by(|a, b| {
-                utils::gen_util::get_ordering(
-                    a.mem_usage_bytes,
-                    b.mem_usage_bytes,
-                    proc_widget_state.is_process_sort_descending,
-                )
-            });
-        }
-        ProcessSorting::MemPercent => {
-            to_sort_vec.sort_by(|a, b| {
-                utils::gen_util::get_ordering(
-                    a.mem_percent_usage,
-                    b.mem_percent_usage,
-                    proc_widget_state.is_process_sort_descending,
-                )
-            });
-        }
-        ProcessSorting::ProcessName => {
-            // Don't repeat if false... it sorts by name by default anyways.
-            if proc_widget_state.is_process_sort_descending {
-                to_sort_vec.sort_by_cached_key(|c| c.name.to_lowercase());
-                if proc_widget_state.is_process_sort_descending {
-                    to_sort_vec.reverse();
-                }
-            }
-        }
-        ProcessSorting::Command => {
-            to_sort_vec.sort_by_cached_key(|c| c.command.to_lowercase());
-            if proc_widget_state.is_process_sort_descending {
-                to_sort_vec.reverse();
-            }
-        }
-        ProcessSorting::Pid => {
-            if !proc_widget_state.is_grouped {
-                to_sort_vec.sort_by(|a, b| {
-                    utils::gen_util::get_ordering(
-                        a.pid,
-                        b.pid,
-                        proc_widget_state.is_process_sort_descending,
-                    )
-                });
-            }
-        }
-        ProcessSorting::ReadPerSecond => {
-            to_sort_vec.sort_by(|a, b| {
-                utils::gen_util::get_ordering(
-                    a.rps_f64,
-                    b.rps_f64,
-                    proc_widget_state.is_process_sort_descending,
-                )
-            });
-        }
-        ProcessSorting::WritePerSecond => {
-            to_sort_vec.sort_by(|a, b| {
-                utils::gen_util::get_ordering(
-                    a.wps_f64,
-                    b.wps_f64,
-                    proc_widget_state.is_process_sort_descending,
-                )
-            });
-        }
-        ProcessSorting::TotalRead => {
-            to_sort_vec.sort_by(|a, b| {
-                utils::gen_util::get_ordering(
-                    a.tr_f64,
-                    b.tr_f64,
-                    proc_widget_state.is_process_sort_descending,
-                )
-            });
-        }
-        ProcessSorting::TotalWrite => {
-            to_sort_vec.sort_by(|a, b| {
-                utils::gen_util::get_ordering(
-                    a.tw_f64,
-                    b.tw_f64,
-                    proc_widget_state.is_process_sort_descending,
-                )
-            });
-        }
-        ProcessSorting::State => {
-            to_sort_vec.sort_by_cached_key(|c| c.process_state.to_lowercase());
-            if proc_widget_state.is_process_sort_descending {
-                to_sort_vec.reverse();
-            }
-        }
-        ProcessSorting::User => to_sort_vec.sort_by(|a, b| match (&a.user, &b.user) {
-            (Some(user_a), Some(user_b)) => utils::gen_util::get_ordering(
-                user_a.to_lowercase(),
-                user_b.to_lowercase(),
-                proc_widget_state.is_process_sort_descending,
-            ),
-            (Some(_), None) => std::cmp::Ordering::Less,
-            (None, Some(_)) => std::cmp::Ordering::Greater,
-            (None, None) => std::cmp::Ordering::Less,
-        }),
-        ProcessSorting::Count => {
-            if proc_widget_state.is_grouped {
-                to_sort_vec.sort_by(|a, b| {
-                    utils::gen_util::get_ordering(
-                        a.group_pids.len(),
-                        b.group_pids.len(),
-                        proc_widget_state.is_process_sort_descending,
-                    )
-                });
-            }
-        }
-    }
-}
-
 pub fn create_input_thread(
-    sender: std::sync::mpsc::Sender<
-        BottomEvent<crossterm::event::KeyEvent, crossterm::event::MouseEvent>,
-    >,
-    termination_ctrl_lock: Arc<Mutex<bool>>,
-) -> std::thread::JoinHandle<()> {
+    sender: Sender<BottomEvent>, termination_ctrl_lock: Arc<Mutex<bool>>,
+) -> JoinHandle<()> {
     thread::spawn(move || {
         let mut mouse_timer = Instant::now();
-        let mut keyboard_timer = Instant::now();
 
         loop {
             if let Ok(is_terminated) = termination_ctrl_lock.try_lock() {
@@ -612,20 +430,44 @@ pub fn create_input_thread(
             if let Ok(poll) = poll(Duration::from_millis(20)) {
                 if poll {
                     if let Ok(event) = read() {
-                        if let Event::Key(key) = event {
-                            if Instant::now().duration_since(keyboard_timer).as_millis() >= 20 {
+                        // FIXME: Handle all other event cases.
+                        match event {
+                            // TODO: Might want to debounce this in the future, or take into account the actual resize
+                            // values. Maybe we want to keep the current implementation in case the resize event might
+                            // not fire... not sure.
+                            Event::Resize(_, _) => {
+                                if sender.send(BottomEvent::Resize).is_err() {
+                                    break;
+                                }
+                            }
+                            Event::Paste(paste) => {
+                                if sender.send(BottomEvent::PasteEvent(paste)).is_err() {
+                                    break;
+                                }
+                            }
+                            Event::Key(key) => {
                                 if sender.send(BottomEvent::KeyInput(key)).is_err() {
                                     break;
                                 }
-                                keyboard_timer = Instant::now();
                             }
-                        } else if let Event::Mouse(mouse) = event {
-                            if Instant::now().duration_since(mouse_timer).as_millis() >= 20 {
-                                if sender.send(BottomEvent::MouseInput(mouse)).is_err() {
-                                    break;
+                            Event::Mouse(mouse) => match mouse.kind {
+                                MouseEventKind::Moved | MouseEventKind::Drag(..) => {}
+                                MouseEventKind::ScrollDown | MouseEventKind::ScrollUp => {
+                                    if Instant::now().duration_since(mouse_timer).as_millis() >= 20
+                                    {
+                                        if sender.send(BottomEvent::MouseInput(mouse)).is_err() {
+                                            break;
+                                        }
+                                        mouse_timer = Instant::now();
+                                    }
                                 }
-                                mouse_timer = Instant::now();
-                            }
+                                _ => {
+                                    if sender.send(BottomEvent::MouseInput(mouse)).is_err() {
+                                        break;
+                                    }
+                                }
+                            },
+                            _ => (),
                         }
                     }
                 }
@@ -635,25 +477,24 @@ pub fn create_input_thread(
 }
 
 pub fn create_collection_thread(
-    sender: std::sync::mpsc::Sender<
-        BottomEvent<crossterm::event::KeyEvent, crossterm::event::MouseEvent>,
-    >,
-    control_receiver: std::sync::mpsc::Receiver<ThreadControlEvent>,
+    sender: Sender<BottomEvent>, control_receiver: Receiver<ThreadControlEvent>,
     termination_ctrl_lock: Arc<Mutex<bool>>, termination_ctrl_cvar: Arc<Condvar>,
     app_config_fields: &app::AppConfigFields, filters: app::DataFilters,
     used_widget_set: UsedWidgets,
-) -> std::thread::JoinHandle<()> {
-    let temp_type = app_config_fields.temperature_type.clone();
+) -> JoinHandle<()> {
+    let temp_type = app_config_fields.temperature_type;
     let use_current_cpu_total = app_config_fields.use_current_cpu_total;
+    let unnormalized_cpu = app_config_fields.unnormalized_cpu;
     let show_average_cpu = app_config_fields.show_average_cpu;
     let update_rate_in_milliseconds = app_config_fields.update_rate_in_milliseconds;
 
     thread::spawn(move || {
         let mut data_state = data_harvester::DataCollector::new(filters);
 
-        data_state.set_collected_data(used_widget_set);
+        data_state.set_data_collection(used_widget_set);
         data_state.set_temperature_type(temp_type);
         data_state.set_use_current_cpu_total(use_current_cpu_total);
+        data_state.set_unnormalized_cpu(unnormalized_cpu);
         data_state.set_show_average_cpu(show_average_cpu);
 
         data_state.init();
@@ -676,19 +517,22 @@ pub fn create_collection_thread(
                         data_state.data.cleanup();
                     }
                     ThreadControlEvent::UpdateConfig(app_config_fields) => {
-                        data_state.set_temperature_type(app_config_fields.temperature_type.clone());
+                        data_state.set_temperature_type(app_config_fields.temperature_type);
                         data_state
                             .set_use_current_cpu_total(app_config_fields.use_current_cpu_total);
+                        data_state.set_unnormalized_cpu(unnormalized_cpu);
                         data_state.set_show_average_cpu(app_config_fields.show_average_cpu);
                     }
                     ThreadControlEvent::UpdateUsedWidgets(used_widget_set) => {
-                        data_state.set_collected_data(*used_widget_set);
+                        data_state.set_data_collection(*used_widget_set);
                     }
                     ThreadControlEvent::UpdateUpdateTime(new_time) => {
                         update_time = new_time;
                     }
                 }
             }
+
+            // TODO: [OPT] this feels like it might not be totally optimal. Hm.
             futures::executor::block_on(data_state.update_data());
 
             // Yet another check to bail if needed...
