@@ -1,18 +1,13 @@
+//! These tests are mostly here just to ensure that invalid results will be caught when passing arguments.
 use assert_cmd::prelude::*;
 use predicates::prelude::*;
-use std::process::Command;
 
-// These tests are mostly here just to ensure that invalid results will be caught when passing arguments...
-
-//======================RATES======================//
-
-fn get_binary_location() -> String {
-    env!("CARGO_BIN_EXE_btm").to_string()
-}
+mod util;
+use util::*;
 
 #[test]
 fn test_small_rate() {
-    Command::new(get_binary_location())
+    btm_command()
         .arg("-C")
         .arg("./tests/empty_config.toml")
         .arg("-r")
@@ -26,21 +21,19 @@ fn test_small_rate() {
 
 #[test]
 fn test_large_default_time() {
-    Command::new(get_binary_location())
+    btm_command()
         .arg("-C")
         .arg("./tests/empty_config.toml")
         .arg("-t")
         .arg("18446744073709551616")
         .assert()
         .failure()
-        .stderr(predicate::str::contains(
-            "set your default value to be at most",
-        ));
+        .stderr(predicate::str::contains("could not parse"));
 }
 
 #[test]
 fn test_small_default_time() {
-    Command::new(get_binary_location())
+    btm_command()
         .arg("-C")
         .arg("./tests/empty_config.toml")
         .arg("-t")
@@ -54,21 +47,19 @@ fn test_small_default_time() {
 
 #[test]
 fn test_large_delta_time() {
-    Command::new(get_binary_location())
+    btm_command()
         .arg("-C")
         .arg("./tests/empty_config.toml")
         .arg("-d")
         .arg("18446744073709551616")
         .assert()
         .failure()
-        .stderr(predicate::str::contains(
-            "set your time delta to be at most",
-        ));
+        .stderr(predicate::str::contains("could not parse"));
 }
 
 #[test]
 fn test_small_delta_time() {
-    Command::new(get_binary_location())
+    btm_command()
         .arg("-C")
         .arg("./tests/empty_config.toml")
         .arg("-d")
@@ -82,22 +73,20 @@ fn test_small_delta_time() {
 
 #[test]
 fn test_large_rate() {
-    Command::new(get_binary_location())
+    btm_command()
         .arg("-C")
         .arg("./tests/empty_config.toml")
         .arg("-r")
         .arg("18446744073709551616")
         .assert()
         .failure()
-        .stderr(predicate::str::contains(
-            "set your update rate to be at most unsigned INT_MAX.",
-        ));
+        .stderr(predicate::str::contains("could not parse"));
 }
 
 #[test]
 fn test_negative_rate() {
     // This test should auto fail due to how clap works
-    Command::new(get_binary_location())
+    btm_command()
         .arg("-C")
         .arg("./tests/empty_config.toml")
         .arg("-r")
@@ -111,33 +100,31 @@ fn test_negative_rate() {
 
 #[test]
 fn test_invalid_rate() {
-    Command::new(get_binary_location())
+    btm_command()
         .arg("-C")
         .arg("./tests/empty_config.toml")
         .arg("-r")
         .arg("100-1000")
         .assert()
         .failure()
-        .stderr(predicate::str::contains("invalid digit"));
+        .stderr(predicate::str::contains("could not parse"));
 }
 
 #[test]
 fn test_conflicting_temps() {
-    Command::new(get_binary_location())
+    btm_command()
         .arg("-C")
         .arg("./tests/empty_config.toml")
         .arg("-c")
         .arg("-f")
         .assert()
         .failure()
-        .stderr(predicate::str::contains(
-            "cannot be used with one or more of the other specified arguments",
-        ));
+        .stderr(predicate::str::contains("cannot be used with"));
 }
 
 #[test]
 fn test_invalid_default_widget_1() {
-    Command::new(get_binary_location())
+    btm_command()
         .arg("-C")
         .arg("./tests/empty_config.toml")
         .arg("--default_widget_type")
@@ -149,7 +136,7 @@ fn test_invalid_default_widget_1() {
 
 #[test]
 fn test_invalid_default_widget_2() {
-    Command::new(get_binary_location())
+    btm_command()
         .arg("-C")
         .arg("./tests/empty_config.toml")
         .arg("--default_widget_type")
@@ -165,7 +152,7 @@ fn test_invalid_default_widget_2() {
 
 #[test]
 fn test_missing_default_widget_type() {
-    Command::new(get_binary_location())
+    btm_command()
         .arg("-C")
         .arg("./tests/empty_config.toml")
         .arg("--default_widget_count")
@@ -176,3 +163,16 @@ fn test_missing_default_widget_type() {
             "The following required arguments were not provided",
         ));
 }
+
+#[test]
+fn test_battery_flag() {
+    if !cfg!(feature = "battery") {
+        btm_command()
+            .arg("--battery")
+            .assert()
+            .failure()
+            .stderr(predicate::str::contains(
+                "'--battery' which wasn't expected",
+            ));
+    }
+}
